@@ -1954,7 +1954,7 @@ xmlParseSGMLCatalogPubid(const xmlChar *cur, xmlChar **id) {
 		"malloc of %d byte failed\n", size);
 	return(NULL);
     }
-    while (xmlIsPubidChar(*cur)) {
+    while (xmlIsPubidChar(*cur) || (*cur == '?')) {
 	if ((*cur == stop) && (stop != ' '))
 	    break;
 	if ((stop == ' ') && (IS_BLANK(*cur)))
@@ -2860,7 +2860,7 @@ xmlInitializeCatalog(void) {
 		    paths = cur;
 		    while ((*cur != 0) && (!IS_BLANK(*cur)))
 			cur++;
-		    path = xmlStrndup((const xmlChar *)paths, cur - paths);
+		    path = (char *) xmlStrndup((const xmlChar *)paths, cur - paths);
 		    if (path != NULL) {
 			*nextent = xmlNewCatalogEntry(XML_CATA_CATALOG, NULL,
 				NULL, BAD_CAST path, xmlCatalogDefaultPrefer);
@@ -2917,7 +2917,7 @@ xmlLoadCatalog(const char *filename)
 
 /**
  * xmlLoadCatalogs:
- * @paths:  a space-separated list of catalog files.
+ * @paths:  a list of directories separated by a colon or a space.
  *
  * Load the catalogs and makes their definitions effective for the default
  * external entity loader.
@@ -2938,7 +2938,7 @@ xmlLoadCatalogs(const char *pathss) {
 	while (IS_BLANK(*cur)) cur++;
 	if (*cur != 0) {
 	    paths = cur;
-	    while ((*cur != 0) && (!IS_BLANK(*cur)))
+	    while ((*cur != 0) && (*cur != ':') && (!IS_BLANK(*cur)))
 		cur++;
 	    path = xmlStrndup((const xmlChar *)paths, cur - paths);
 	    if (path != NULL) {
@@ -2946,6 +2946,8 @@ xmlLoadCatalogs(const char *pathss) {
 		xmlFree(path);
 	    }
 	}
+	while (*cur == ':')
+	    cur++;
     }
 }
 
