@@ -6,19 +6,14 @@
  *
  * daniel@veillard.com
  *
- * 14 Nov 2000 ht - added redefinition of xmlBufferWriteChar for VMS
- *
  */
 
 #ifndef __XML_TREE_H__
 #define __XML_TREE_H__
 
 #include <stdio.h>
-#if defined(WIN32) && (defined(_MSC_VER) || defined(__BORLANDC__))
-#include <libxml/xmlwin32version.h>
-#else
 #include <libxml/xmlversion.h>
-#endif
+
 #ifdef __cplusplus
 extern "C" {
 #endif
@@ -799,14 +794,8 @@ int		xmlUnsetNsProp		(xmlNodePtr node,
 /*
  * Internal, don't use.
  */
-#ifdef VMS
-void		xmlBufferWriteXmlCHAR	(xmlBufferPtr buf,
-					 const xmlChar *string);
-#define 	xmlBufferWriteCHAR 	xmlBufferWriteXmlCHAR
-#else
 void		xmlBufferWriteCHAR	(xmlBufferPtr buf,
 					 const xmlChar *string);
-#endif
 void		xmlBufferWriteChar	(xmlBufferPtr buf,
 					 const char *string);
 void		xmlBufferWriteQuotedString(xmlBufferPtr buf,
@@ -879,6 +868,12 @@ int		xmlSaveFileEnc		(const char *filename,
 					 xmlDocPtr cur,
 					 const char *encoding);
 
+/*
+ * XHTML
+ */
+int		xmlIsXHTML		(const xmlChar *systemID,
+					 const xmlChar *publicID);
+
 /*
  * Compression.
  */
